@@ -43,10 +43,6 @@
 #include <stdlib.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-client.h>
-#include <libgnomeui/gnome-app-helper.h>
-#include <libgnomeui/gnome-authentication-manager.h>
 
 #if HAVE_EXEMPI
 #include <exempi/xmp.h>
@@ -174,9 +170,12 @@ load_files_remote (void)
 int
 main (int argc, char **argv)
 {
-	GnomeProgram *program;
+	GError *error = NULL;
 	GOptionContext *ctx;
 
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
 	bindtextdomain (PACKAGE, EOG_LOCALE_DIR);
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
@@ -185,13 +184,27 @@ main (int argc, char **argv)
 
 	ctx = g_option_context_new (NULL);
 	g_option_context_add_main_entries (ctx, goption_options, PACKAGE);
+	/* Option groups are free'd together with the context */
+	g_option_context_add_group (ctx, gtk_get_option_group (TRUE));
+
+	gtk_init (&argc, &argv);
+
+	if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
+		gchar *help_msg;
+
+		/* I18N: The '%s' is replaced with eog's command name. */ 
+		help_msg = g_strdup_printf (_("Run '%s --help' to see a full "
+					      "list of available command line "
+					      "options."), argv[0]);
+                g_print("%s\n%s\n", error->message, help_msg); 
+                g_error_free (error);
+		g_free (help_msg);
+                g_option_context_free (ctx);
+                
+                return 1;
+        }
+	g_option_context_free (ctx);
 
-	program = gnome_program_init ("eog", VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_GOPTION_CONTEXT, ctx,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Eye of GNOME"),
-				      GNOME_PARAM_APP_DATADIR, EOG_DATA_DIR,
-				      NULL);
 	
 	set_startup_flags ();
 	
@@ -199,14 +212,11 @@ main (int argc, char **argv)
 	if (!force_new_instance &&
 	    !eog_application_register_service (EOG_APP)) {
 		if (load_files_remote ()) {
-			g_object_unref (program);
 			return 0;
 		}
 	}
 #endif /* HAVE_DBUS */
 	
-	gnome_authentication_manager_init ();
-
 #ifdef HAVE_EXEMPI
  	xmp_init();
 #endif
@@ -231,15 +241,11 @@ main (int argc, char **argv)
 
 	gdk_threads_leave ();
 
-	gnome_accelerators_sync ();
-	
   	if (startup_files)
 		g_strfreev (startup_files);
 
 	eog_plugin_engine_shutdown ();
 
-	g_object_unref (program);
-
 #ifdef HAVE_EXEMPI
 	xmp_terminate();
 #endif
